/*
 *  sdspdemo.c - demonstrate the usage of the SDSP System.
 *
 *    by Joel David Miller
 *
 *  modification history
 *  --------------------
 *  5/23/94 	JDM	created
*/

#import		<stdio.h>
#import		<sound/soundstruct.h>
#import		<sound/filesound.h>
#import		<sound/utilsound.h>
#import		<sound/sounderror.h>

// NEXT2_1: 1 = NeXTstep 2.1, 0 = NeXTstep 3.1
#define	NEXT2_1	0
#if NEXT2_1
  #import	<mach.h>
#else
  #import	<mach/mach.h>
#endif

#import		"mydsp.h"
#import		"sdspsys_internal.h"

#define	VIEW		8
#define	BUF_PTS		100
#define	MOD_PATH	"/Common/Projects/DSP/sdsp/fft_example/mods/"

  /* SDSP_ERR - prints an error message, deallocates ports */
#define	SDSP_ERR							\
{									\
  printf( "\nError: %s.\n", SDSPErrorStr( sdsp_error ) );		\
  SDSPFree();								\
  exit(1);								\
}

#include<stdio.h>

  /* Global vars */
SNDSoundStruct		*sound = 0;
MyDSPStruct		*freq = 0;


/*----------------------------------------------------------------------*/
/* sfft() - demonstrate MyDSPSFFT					*/

SDSPError
demo_sfft( void )
{
  SDSPError	sdsp_error;  
  int		frame_pts;
  int		frame_inc;
  int		*data_r;
  int		*data_i;
  int		snd_error;
  int		i,j;
  char		input[ BUF_PTS ];

  puts( "\n---- Demonstrating MyDSPSFFT() ----" );

  fputs( "\nEnter the name of a sound (.snd) file: ", stdout );
  fgets( input, BUF_PTS, stdin );
  input[ strlen(input)-1 ] = '\0';

  if( snd_error = SNDReadSoundfile( input, &sound ) )
  {
    fprintf( stderr, "\n** Error - reading sound file [%s]\n", input );
    fprintf( stderr, "     SNDSoundError: [%8x]  [%s]\n\n", snd_error,
    SNDSoundError( snd_error ) );
    SDSPFree();
    exit(1);
  }

  fputs( "Enter the number of points in a frame: ", stdout );
  fgets( input, BUF_PTS, stdin );
  frame_pts = atoi( input );

  fputs( "Enter the frame increment: ", stdout );
  fgets( input, BUF_PTS, stdin );
  frame_inc = atoi( input );

  if( freq )
  {
    vm_deallocate( task_self(), (vm_address_t) freq->Data1,
      freq->FramePts * freq->FrameNum * 2 * sizeof( int ) );
    free( freq );
  }

  if( sdsp_error = MyDSPSFFT( sound, &freq, frame_pts, frame_inc ) )
  {
    fputs( "\n** Error while computing the sound fft :", stderr );
    return sdsp_error;
  }

  puts( "\n\n**** SFFT DATA ****" );

  data_r = freq->Data1;
  data_i = data_r + freq->FramePts;
  for( i = 0; i < freq->FrameNum; ++i )
  {
    printf( "\n**  FRAME #%d  **\n\n", i );
    printf( "Frequency Scalar = %d  0x%08x\n\n", freq->Scalar[i],
            freq->Scalar[i] );

    for( j = 0; j < VIEW; ++j )
      printf( "INDEX: [%4d]    FREQ: R [%8d]  I [%8d]\n", j, data_r[j],
              data_i[j] );
    puts( "..." );
    for( j = freq->FramePts - VIEW; j < freq->FramePts; ++j )
      printf( "INDEX: [%4d]    FREQ: R [%8d]  I [%8d]\n", j, data_r[j],
              data_i[j] );

    data_r += 2 * freq->FramePts;
    data_i += 2 * freq->FramePts;
  }

  SNDFree( sound );
  return SDSP_ERR_NONE;
}


/*----------------------------------------------------------------------*/
/* main()								*/
/*----------------------------------------------------------------------*/

void
main( argc, argv )
int	argc;
char	*argv[];
{
  SDSPError		sdsp_error;
  char			input[ BUF_PTS ];


  puts( "\n\n****  SDSP System Demonstration  ****" );

  printf( "\nDefault module path: \"%s\"\n", MOD_PATH );
  if( sdsp_error = SDSPModPath( MOD_PATH ) )  SDSP_ERR;

  while(1)
  {
puts
(
"\n\n"
"-------------------------- OPTIONS --------------------------\n"
"[ 1 ]  MyDSPSFFT()       ---- run demonstration function\n"
"\n"
"[ 2 ]  SDSPPrintStruct() ---- print SDSP internal information\n"
"[ 3 ]  SDSPModPath()     ---- change the module path\n"
"[ 4 ]  SDSPOwned()       ---- check DSP ownership\n"
"[ 5 ]  SDSPVersion()     ---- print hardware version\n"
"[ 6 ]  SDSPFreeDSP()     ---- free the DSP for external use\n"
"[ 7 ]  SDSPFree()        ---- free all SDSP ports, data, etc.\n"
"[ 8 ]  exit demo         ---- return to command line"
);
    fputs( "\nEnter option: ", stdout );
    fgets( input, BUF_PTS, stdin );

    switch( input[0] )
    {
      case '1':		/* **** 1 **** - MyDSPSFFT()		*/
        if( sdsp_error = demo_sfft() )  SDSP_ERR;
        break;

      case '2':		/* **** 2 **** - SDSPPrintStruct()	*/
        SDSPPrintStruct();
        break;

      case '3':		/* **** 3 **** - SDSPModPath()		*/
        fputs( "\nEnter the name of the module path: ", stdout );
	fgets( input, BUF_PTS, stdin );
	input[ strlen( input ) - 1 ] = '\0';	/* kill '\n' */
	if( sdsp_error = SDSPModPath( input ) )  SDSP_ERR;
        break;

      case '4':		/* **** 4 **** - SDSPOwned()		*/
        printf( "\n  SDSPOwned() = %d\n", SDSPOwned() );
        break;

      case '5':		/* **** 5 **** - SDSPVersion()		*/
        if( SDSPVersion() == 1 )
          puts( "\n  24-bit DMA not functioning.\n" );
        else
          puts( "\n  24-bit DMA functioning.\n" );
        break;

      case '6':		/* **** 6 **** - SDSPFreeDSP()		*/
        if( sdsp_error = SDSPFreeDSP() )  SDSP_ERR;
        break;

      case '7':		/* **** 7 **** - SDSPFree()		*/
        if( sdsp_error = SDSPFree() )  SDSP_ERR;
        break;

      case 'q':
      case '8':		/* **** 8 **** - exit demo		*/
        puts( "\n" );
        exit(1);
        break;

      default:
        break;
    }
  }
}
