/*
 *  sdspsys_internal.h - this file should only be included by code which
 *    uses the internals of the SDSP system (e.g. sdspsys.c or debug code).
 *
 *  by Joel David Miller
 *
 *  modification history
 *  --------------------
 *  1/21/93	JDM	created
 *  1/25/93	JDM	added imports which used to be sdspsys.h
 *  2/11/93	JDM	separated SDSP and SPED DSP errors
 *  6/22/93	JDM	moved SDSPInitPorts() and SDSPSetProtocol() from
 *			sdspsys.h to here
 *  6/30/93	JDM	added SDSP_SYS_STREAM_EN and SDSP_SYS_STREAM_DIS
 *  7/9/93	JDM	removed #import <dsp/arrayproc.h> and
 *			#import <dsp/DSPConversion.h>
 *  7/12/93	JDM	added define for CVR_HC
 *  7/27/93	JDM	added status field to HandArg structure,
 *			SDSP_READ_COMPLETE, and SDSP_WRITE_COMPLETE
 *  8/3/93	JDM	added SDSP_MSG_READ_COMPLETE, SDSP_READ_CLEAR,
 *			SDSP_WRITE_CLEAR, and SDSP_MSG_READ_CLEAR;
 *			added SDSPDebugMsg()
*/

/* 1 = NeXTstep 2.1, 0 = NeXTstep 3.1 */
#if 0
  #import	<nextdev/snd_dspreg.h>		/* CVR_HC		*/
#else
  #define	CVR_HC	0x80			/* CVR_HC		*/
#endif

#import <sound/sounddriver.h>		/* snddriver functions, etc.	*/
#import	<sound/sound.h>

  /* flags for SDSPSysStruct.status */
#define	SDSP_SYS_DSP_OWNED	0x00000001	/* DSP owned by SDSP system */
#define SDSP_SYS_DSP_FREE	0xFFFFFFFE	/* DSP free		    */
#define	SDSP_SYS_STREAM_EN	0x00000002	/* streams enabled	    */
#define	SDSP_SYS_STREAM_DIS	0xFFFFFFFD	/* streams disabled	    */

#define	SDSP_STREAM_READ	0x00000001	/* wait for a stream read   */
#define	SDSP_STREAM_WRITE	0x00000002	/* wait for a stream write  */
#define	SDSP_MESSAGE_READ	0x00000004	/* wait for a message read  */

#define	SDSP_READ_COMPLETE	0x00000001	/* stream read complete	    */
#define	SDSP_WRITE_COMPLETE	0x00000002	/* stream write complete    */
#define	SDSP_MSG_READ_COMPLETE	0x00000004	/* message read complete    */

#define	SDSP_STREAM_READ_CLR	0xFFFFFFFE	/* clear stream read	    */
#define	SDSP_STREAM_WRITE_CLR	0xFFFFFFFD	/* clear stream write	    */
#define	SDSP_MESSAGE_READ_CLR	0xFFFFFFFB	/* clear message read	    */

#define	SDSP_READ_CLEAR		0xFFFFFFFE	/* clear stream read	    */
#define	SDSP_WRITE_CLEAR	0xFFFFFFFD	/* clear stream write	    */
#define	SDSP_MSG_READ_CLEAR	0xFFFFFFFB	/* clear message read	    */

#define	READ_TAG24	3	/* identifies the region/stream		    */
#define	LOW_WATER	49152	/* 48k minimum amount wired down 	    */
#define	HIGH_WATER	65536	/* 64k wired down upon hitting LOW_WATER    */

  /* struct containing the SDSP system control variables */
typedef struct
{
  char		*mod_path;	/* path for the DSP subroutine modules	    */
  int		mod_num;	/* current module loaded to the DSP	    */
  port_t	dev_port;	/* device port obtained from SNDAcquire()   */
  port_t	owner_port;	/* owner port obtained from SNDAcquire()    */
  port_t	cmd_port;	/* command port for issuing commands to DSP */
  port_t	reply_port;	/* port for receiving reply messages	    */
  port_t	write_port;	/* identifier for the stream to the DSP	    */
  int		w_buf_pts;	/* number of pts in the write stream buffer */
  int		w_data_width;	/* number of bytes in one data element	    */
  int		w_req_width;	/* width requested - used by stream write   */
  port_t	read_port;	/* identifier for the stream from the DSP   */
  int		r_buf_pts;	/* number of pts in the read stream buffer  */
  int		r_data_width;	/* number of bytes in one data element	    */
  int		r_req_width;	/* width requested - used by stream read    */
  int		protocol;	/* method of DSP communication		    */
  int		version;	/* broken 24-bit DMA (1) or fixed (2)	    */
  int		status;		/* current status of the SDSP system	    */
} SDSPSysStruct;

#if 0
  /* argument to the handler for stream reads */
typedef struct
{
  int		status;
  int		done;
  int		*read_data;
  int		bytes_read;
} HandArg;
#endif
  /* argument to the handler for stream reads */
typedef struct
{
  int		status;
  int		done;
  int		*read_data;
  int		bytes_read;
  int		*message_data;
  int		message_index;
  int		message_expected;
  int		message_enqueue;
} HandArg;

/************************************************************************/
/* INITIALIZATION							*/
/************************************************************************/

extern	SDSPError	SDSPInitPorts();

/************************************************************************/
/* STREAMED I/O								*/
/************************************************************************/

extern	SDSPError	SDSPSetProtocol( void );

/************************************************************************/
/* ERROR								*/
/************************************************************************/

extern	int		n_gen_err;

#if 0
  /* mainly for debugging purposes - see sdsperror.c */
void	err_hand();	/* stream i/o error handling function	 */
#endif

/************************************************************************/
/* DEBUG								*/
/************************************************************************/

extern	void	SDSPPrintStruct( void );
extern	void	SDSPDebugMsg( char *debug_message, msg_header_t *msg );

