/*
 *  sdspsys_waitmacro.h - SDSPWaitQ_IO_BEGIN() and SDSPWaitQ_IO_END().
 *
 *  by Joel David Miller
 *
 *  modification history
 *  --------------------
 *  6/30/93	JDM	created
 *  7/9/93	JDM	made error reporting more robust
 *  7/27/93	JDM	added SDSPWaitRead() and SDSPWaitWrite()
 *  8/4/93	JDM	altered everything to rely on SDSPWaitQ_IO()
*/

#import	"sdspsys.h"
#import	"sdspsys_internal.h"
#import	"sdspsys_error.h"	/* SDSPError	*/

  /* global variables from sdspsys.c */
extern	SDSPSysStruct		sys;
extern	snddriver_handlers_t	handlers;
extern	HandArg			from_dsp;
extern	msg_header_t		*reply_msg;

#define	DEBUG_IO	0


/*----------------------------------------------------------------------*/
/* SDSPWaitQ_IO() - waits for the current enqueued I/O activity to	*/
/*   complete by receiving messages which are then dispatched to the	*/
/*   appropriate handler.						*/

#define	SDSPWaitQ_IO_BEGIN( io_flags, err )				\
  err = SDSP_ERR_NONE;							\
  if( !(sys.status & SDSP_SYS_STREAM_EN) )  err = SDSP_ERR_STREAM_DIS;	\
  handlers.arg = &from_dsp;						\
  if( (io_flags) & SDSP_STREAM_READ )					\
    from_dsp.status &= SDSP_STREAM_READ_CLR;				\
  if( (io_flags) & SDSP_STREAM_WRITE )					\
    from_dsp.status &= SDSP_STREAM_WRITE_CLR;				\
  if( (io_flags) & SDSP_MESSAGE_READ )					\
    from_dsp.status &= SDSP_MESSAGE_READ_CLR;				\
  while(1)								\
  {

#define	SDSPWaitQ_IO_END( io_flags, err )				\
    if( (from_dsp.status & (io_flags)) == (io_flags) )  break;		\
    reply_msg->msg_size = MSG_SIZE_MAX;					\
    reply_msg->msg_local_port = sys.reply_port;				\
    n_gen_err = msg_receive( reply_msg, MSG_OPTION_NONE, 0 );		\
    if( DEBUG_IO )  SDSPDebugMsg( "** Wait I/O message:", reply_msg );	\
    if( n_gen_err != KERN_SUCCESS )					\
    {									\
      err = SDSP_ERR_R_MSG;						\
    }									\
    else								\
    {									\
      n_gen_err = snddriver_reply_handler( reply_msg, &handlers );	\
      if( n_gen_err )  err = SDSP_ERR_P_MSG;				\
    }									\
  }

